@@ -73,7 +73,7 @@ def import_data(sbdf_file: typing.Union[str, bytes, int]) -> pd.DataFrame:
         table_metadata_dict = _import_table_metadata(tmeta)
 
         # Process column metadata
-        pd_data, pd_dtypes, column_metadata_dict, column_names = _import_column_metadata(tmeta)
+        pd_data, pd_dtypes, spotfire_types, column_metadata_dict, column_names = _import_column_metadata(tmeta)
 
         # Read the table slices
         _import_table_slices(file, column_names, pd_data, tmeta)
@@ -85,6 +85,7 @@ def import_data(sbdf_file: typing.Union[str, bytes, int]) -> pd.DataFrame:
         dataframe = pd.concat(columns, axis=1)
         for col in column_names:
             dataframe[col].spotfire_column_metadata = column_metadata_dict[col]
+            dataframe[col].attrs['spotfire_type'] = spotfire_types[col]
         if gpd is not None and table_metadata_dict.get('MapChart.IsGeocodingTable'):
             dataframe = _data_frame_to_geo_data_frame(dataframe, table_metadata_dict)
         with warnings.catch_warnings():
@@ -101,11 +102,13 @@ def _import_table_metadata(tmeta: '_TableMetadata') -> typing.Dict[str, typing.A
 
 
 def _import_column_metadata(tmeta: '_TableMetadata') -> typing.Tuple[typing.Dict[str, typing.List],
+                                                                     typing.Dict[str, str],
                                                                      typing.Dict[str, str],
                                                                      typing.Dict[str, typing.Dict[str, typing.Any]],
                                                                      typing.List[str]]:
     pd_data = {}
     pd_dtypes = {}
+    spotfire_types = {}
     column_metadata_dict = {}
     column_names = []
     for i in range(tmeta.column_count()):
@@ -118,8 +121,12 @@ def _import_column_metadata(tmeta: '_TableMetadata') -> typing.Tuple[typing.Dict
         # add a new list to hold the column
         pd_data[cm_column_name] = []
 
+        # get the Spotfire type for the column
+        sbdf_type = _ColumnMetadata.get_type(cmeta)
+        spotfire_types[cm_column_name] = sbdf_type.to_typename_string()
+
         # get the pandas dtype for constructing this column
-        pd_dtypes[cm_column_name] = _ColumnMetadata.get_type(cmeta).to_dtype_name()
+        pd_dtypes[cm_column_name] = sbdf_type.to_dtype_name()
 
         # get the remaining column metadata
         cm_dict = {}
@@ -129,7 +136,7 @@ def _import_column_metadata(tmeta: '_TableMetadata') -> typing.Tuple[typing.Dict
             cm_dict[cmeta.names[j]] = cmeta.values[j].data
         column_metadata_dict[cm_column_name] = cm_dict
 
-    return pd_data, pd_dtypes, column_metadata_dict, column_names
+    return pd_data, pd_dtypes, spotfire_types, column_metadata_dict, column_names
 
 
 def _import_table_slices(file: typing.BinaryIO, column_names: typing.List[str],
@@ -173,7 +180,10 @@ def export_data(obj: typing.Any, sbdf_file: typing.Union[str, bytes, int], defau
         tmeta.write(file)
 
         # Write out the table and column slices
-        _export_table_slices(columns, column_names, column_types, file, row_count, tmeta)
+        try:
+            _export_table_slices(columns, column_names, column_types, file, row_count, tmeta)
+        except Exception as exc:
+            raise SBDFError("Failure while exporting data") from exc
 
 
 def _export_columnize_data(obj: typing.Any, default_column_name: str) -> \
@@ -966,6 +976,24 @@ class _ValueTypeId(enum.IntEnum):
             _ValueTypeId.DECIMAL: "Currency"
         }.get(self, "unknown")
 
+    @staticmethod
+    def from_typename_string(typename: str) -> '_ValueTypeId':
+        """convert the type name used by Spotfire to a valuetype id"""
+        return {
+            "Boolean": _ValueTypeId.BOOL,
+            "Integer": _ValueTypeId.INT,
+            "LongInteger": _ValueTypeId.LONG,
+            "SingleReal": _ValueTypeId.FLOAT,
+            "Real": _ValueTypeId.DOUBLE,
+            "DateTime": _ValueTypeId.DATETIME,
+            "Date": _ValueTypeId.DATE,
+            "Time": _ValueTypeId.TIME,
+            "TimeSpan": _ValueTypeId.TIMESPAN,
+            "String": _ValueTypeId.STRING,
+            "Binary": _ValueTypeId.BINARY,
+            "Currency": _ValueTypeId.DECIMAL
+        }.get(typename)
+
     def to_dtype_name(self) -> str:
         """convert this valuetype id to the dtype name used by Pandas"""
         return {
@@ -1013,6 +1041,23 @@ class _ValueTypeId(enum.IntEnum):
     @staticmethod
     def infer_from_dtype(series: pd.Series, series_description: str) -> '_ValueTypeId':
         """determine the proper valuetype id from the Pandas dtype of a series"""
+        # use explicitly set type info if present
+        sbdf_type = None
+        if 'spotfire_type' in series.attrs:
+            sbdf_type = _ValueTypeId.from_typename_string(series.attrs['spotfire_type'])
+            if sbdf_type == _ValueTypeId.INT:
+                int32_info = np.iinfo(np.int32)
+                for i in series:
+                    if isinstance(i, int) and not int32_info.min < i < int32_info.max:
+                        warnings.warn(
+                            f"values in {series_description} do not fit in type 'Integer'; promoting type to "
+                            f"'LongInteger'")
+                        sbdf_type = _ValueTypeId.LONG
+                        break
+        if sbdf_type:
+            return sbdf_type
+
+        # otherwise, infer the type info
         dtype = series.dtype.name
         if dtype == "object":
             return _ValueTypeId.infer_from_type(series.tolist(), series_description)
@@ -1206,7 +1251,7 @@ class _ValueType:
 
     @staticmethod
     def _to_bytes_string(obj: str) -> bytes:
-        return obj.encode("utf-8")
+        return str(obj).encode("utf-8")
 
     @staticmethod
     def _to_bytes_binary(obj: bytes) -> bytes:
@@ -1235,7 +1280,10 @@ class _ValueType:
             return getattr(self, "_to_bytes_" + self.type_id.name.lower(), lambda x: None)(obj)
         except (struct.error, bitstring.CreationError, UnicodeError) as exc:
             raise SBDFError(f"cannot convert '{obj}' to Spotfire {self.type_id.to_typename_string()} \
-            type; value is outside representable range") from exc
+                        type; value is outside representable range") from exc
+        except (AttributeError, TypeError) as exc:
+            raise SBDFError(f"cannot convert '{obj}' to Spotfire {self.type_id.to_typename_string()} \
+                        type; incompatible types") from exc
 
     def missing_value(self) -> typing.Any:
         """return a missing value appropriate for the value type"""
@@ -1436,6 +1484,10 @@ if gpd is not None:
         if not isinstance(gdf.get("geometry")[0], shp_geom.BaseGeometry):
             return gdf
 
+        # Save CRS before manipulating gdf, as geopandas >= 0.11.0 disallows fetching it
+        # after dropping the geometry column
+        saved_crs = gdf.crs
+
         # Write geocoding column metadata
         cols = ["XMin", "XMax", "YMin", "YMax", "XCenter", "YCenter", "Geometry"]
         metacols = list(gdf.columns.drop('geometry'))
@@ -1478,14 +1530,14 @@ if gpd is not None:
             table_metadata["MapChart.GeometryType"] = "Polygon"
         else:
             raise SBDFError("cannot convert collections of Shapely objects")
-        if gdf.crs is not None:
+        if saved_crs is not None:
             try:
-                table_metadata["MapChart.GeographicCrs"] = gdf.crs.to_string()
+                table_metadata["MapChart.GeographicCrs"] = saved_crs.to_string()
             except AttributeError:
                 # GeoPandas <= 0.6.3 compatibility
-                if gdf.crs.startswith("+init="):
-                    gdf.crs = gdf.crs[6:]
-                table_metadata["MapChart.GeographicCrs"] = gdf.crs
+                if saved_crs.startswith("+init="):
+                    saved_crs = saved_crs[6:]
+                table_metadata["MapChart.GeographicCrs"] = saved_crs
         return dframe
 
 
